@@ -1,7 +1,7 @@
 from typing import Tuple, Union
 
 from nonebot import CommandGroup
-from nonebot.adapters import Bot, Event
+from nonebot.adapters import Bot
 from nonebot.adapters.cqhttp import MessageEvent, GroupMessageEvent
 
 from . import core
@@ -16,24 +16,40 @@ def h(x):
     return x.handle()
 
 
-def plaintext(_b, e, _s):
-    return all(seg.is_text() for seg in e.get_message())
+async def ensure_command(_b, e, s):
+    msg = e.get_message()
+    raw_cmd = s["_prefix"]["raw_command"]
+    if not msg or not raw_cmd or not all(seg.is_text() for seg in msg):
+        return False
+
+    first_seg = str(msg[0]).lstrip()
+    if not first_seg.startswith(raw_cmd):
+        return False
+    return (len(msg) == 1 and len(first_seg) == len(raw_cmd)  # 无参数
+            or first_seg[len(raw_cmd)].isspace())  # 命令名后有空格
 
 
-@h(cg.command('reload', permission=P('reload')))
-async def _(bot: Bot, event: Event):
-    core.reload()
-    await bot.send(event, '重新加载权限配置')
-
-
-@h(cg.command('save', permission=P('reload')))
+@h(cg.command('reload', rule=ensure_command, permission=P('reload')))
 async def _(bot: Bot, event: MessageEvent):
-    core.save_all()
-    await bot.send(event, '已保存权限配置')
+    force = str(event.message).strip() == 'force'
+    reloaded = core.reload(force)
+    if reloaded:
+        await bot.send(event, '重新加载权限配置')
+    else:
+        await bot.send(event, '有未保存的修改，如放弃修改请添加force参数')
 
 
-@h(cg.command('add', rule=plaintext, permission=P('edit.perm'), state={'add': True}))
-@h(cg.command('remove', rule=plaintext, permission=P('edit.perm'), state={'add': False}))
+@h(cg.command('save', rule=ensure_command, permission=P('reload')))
+async def _(bot: Bot, event: MessageEvent):
+    success = core.save_all()
+    if success:
+        await bot.send(event, '已保存权限配置')
+    else:
+        await bot.send(event, '部分配置保存失败，请检查控制台输出')
+
+
+@h(cg.command('add', rule=ensure_command, permission=P('edit.perm'), state={'add': True}))
+@h(cg.command('remove', rule=ensure_command, permission=P('edit.perm'), state={'add': False}))
 async def _(bot: Bot, event: MessageEvent, state: dict):
     args = str(event.message).split()
 
@@ -58,24 +74,25 @@ async def _(bot: Bot, event: MessageEvent, state: dict):
 
     try:
         if state['add']:
-            P.add_item(namespace, group, item)
+            result = P.add_item(namespace, group, item)
         else:
-            P.remove_item(namespace, group, item)
+            result = P.remove_item(namespace, group, item)
     except TypeError:
         await bot.send(event, '权限组不可修改')
     except KeyError:
         await bot.send(event, '权限组不存在')
-    except ValueError:
-        await bot.send(event, '权限组中{}指定描述'.format('已有' if state['add'] else '没有'))
     else:
-        await bot.send(event, '已修改权限组')
+        if result:
+            await bot.send(event, '已修改权限组')
+        else:
+            await bot.send(event, '权限组中{}指定描述'.format('已有' if state['add'] else '没有'))
 
 
-@h(cg.command('addgrp', rule=plaintext, permission=P('edit.group'), state={'add': True}))
-@h(cg.command('rmgrp', rule=plaintext, permission=P('edit.group'), state={'add': False, 'force': False}))
-@h(cg.command('rmgrpf', rule=plaintext, permission=P('edit.group.force'), state={'add': False, 'force': True}))
+@h(cg.command('addgrp', rule=ensure_command, permission=P('edit.group'), state={'add': True}))
+@h(cg.command('rmgrp', rule=ensure_command, permission=P('edit.group'), state={'add': False, 'force': False}))
+@h(cg.command('rmgrpf', rule=ensure_command, permission=P('edit.group.force'), state={'add': False, 'force': True}))
 async def _(bot: Bot, event: MessageEvent, state: dict):
-    arg = str(event.message)
+    arg = str(event.message).strip()
 
     # 无参数，编辑当前会话权限组
     if not arg:
